<?php
/*
 * PHP Multicolor Picture Shower
 * Coded By: Skate / Glance
 * 25 August 2008
 */

// Turn Magic Quotes Off For Safe Binary File Reading
set_magic_quotes_runtime(0);

// Constant Definitions
define("GRAPHIC_MODE", 0);		// 0 = Koala
								// 1 = Advanced Art Studio

define("HEADER_LENGTH", 2);		// PRG files have 2 header bytes
								// If it's a flat binary file,
								// HEADER_LENGTH should be zero

define("ARRAY_INDEX_SEED", 1);	// "unpack" command creates an array
								// starting from 1 not 0

define("DATA_POINTER", HEADER_LENGTH + ARRAY_INDEX_SEED);

switch(GRAPHIC_MODE) {
	case 0: // Koala
		define("BITMAP",	DATA_POINTER);
		define("VIDEO",		DATA_POINTER + 8000);
		define("COLOR",		DATA_POINTER + 9000);
		define("BGCOLOR",	DATA_POINTER + 10000);
		$filename = "./samples/koala.prg";
		break;
	case 1: // Advanced Art Studio
		define("BITMAP",	DATA_POINTER);
		define("VIDEO",		DATA_POINTER + 8000);
		define("COLOR",		DATA_POINTER + 9016);
		define("BGCOLOR",	DATA_POINTER + 9001);
		$filename = "./samples/adv_art_studio.prg";
		break;
}

// Load Picture
$handle = @fopen($filename, "rb") or die("Cannot open file");
$picBinary = unpack("C*",fread($handle, filesize($filename)));
fclose($handle);

// Create Image
header ("Content-type: image/png");
$pic = @imagecreatetruecolor(320, 200) or die("Cannot init image stream");

// Prepare C64 Palette (taken from Vice : c64/vice.vpl)
$c64Black		= imagecolorallocate($pic, 0x00, 0x00, 0x00);
$c64White		= imagecolorallocate($pic, 0xff, 0xff, 0xff);
$c64Red			= imagecolorallocate($pic, 0x68, 0x37, 0x2b);
$c64Cyan		= imagecolorallocate($pic, 0x70, 0xa4, 0xb2);
$c64Purple		= imagecolorallocate($pic, 0x6f, 0x3d, 0x86);
$c64Green		= imagecolorallocate($pic, 0x58, 0x8d, 0x43);
$c64Blue		= imagecolorallocate($pic, 0x35, 0x28, 0x79);
$c64Yellow		= imagecolorallocate($pic, 0xb8, 0xc7, 0x6f);
$c64Orange		= imagecolorallocate($pic, 0x6f, 0x4f, 0x25);
$c64Brown		= imagecolorallocate($pic, 0x43, 0x39, 0x00);
$c64LightRed	= imagecolorallocate($pic, 0x9a, 0x67, 0x59);
$c64DarkGray	= imagecolorallocate($pic, 0x44, 0x44, 0x44);
$c64MediumGray	= imagecolorallocate($pic, 0x6c, 0x6c, 0x6c);
$c64LightGreen	= imagecolorallocate($pic, 0x9a, 0xd2, 0x84);
$c64LightBlue	= imagecolorallocate($pic, 0x6c, 0x5e, 0xb5);
$c64LightGray	= imagecolorallocate($pic, 0x95, 0x95, 0x95);

// Create A C64 Color Array For Indexed Access
$c64Colors = array($c64Black, $c64White, $c64Red, $c64Cyan, $c64Purple, $c64Green, $c64Blue, $c64Yellow, $c64Orange, $c64Brown, $c64LightRed, $c64DarkGray, $c64MediumGray, $c64LightGreen, $c64LightBlue, $c64LightGray);

// Get Background Color
$bgColorIndex = $picBinary[BGCOLOR];

// Prepare Color Array For Character Areas
$charColors = array($c64Colors[$bgColorIndex], null, null, null);

// Generate Image
for($col = 0; $col < 40; $col++) {
	for($row = 0; $row < 25; $row++) {
		// Precalculations
		$rowPre1 = ($row << 8) + ($row << 6); // $row * 320
		$rowPre2 = ($row << 5) + ($row << 3); // $row * 40
		$rowPre3 = $row << 3; // $row * 8
		$colPre = $col << 3; // $col * 8

		$charAddress = $rowPre2 + $col; // $row * 40 + $col

		// Find Colors Of Character Block
		$charColors[1] = $c64Colors[$picBinary[VIDEO+$charAddress] >> 4]; // get high nibble of video mem
		$charColors[2] = $c64Colors[$picBinary[VIDEO+$charAddress] & 15]; // get low nibble of video mem
		$charColors[3] = $c64Colors[$picBinary[COLOR+$charAddress] & 15]; // get low nibble of color mem

		// Draw Character
		for($charRow = 0; $charRow < 8; $charRow++) {
			// Get Byte Value
			$byteValue = $picBinary[BITMAP + $rowPre1 + $colPre + $charRow];

			// Find Colors Of Bit Doubles
			$color1 = $charColors[$byteValue >> 6];			// %XX000000 -> %XX
			$color2 = $charColors[($byteValue >> 4) & 3];	// %00XX0000 -> %XX
			$color3 = $charColors[($byteValue >> 2) & 3];	// %0000XX00 -> %XX
			$color4 = $charColors[$byteValue & 3];			// %000000XX -> %XX

			// Draw Pixels Of A Character Row
			imagesetpixel($pic, $colPre  , $rowPre3 + $charRow, $color1);
			imagesetpixel($pic, $colPre+1, $rowPre3 + $charRow, $color1);
			imagesetpixel($pic, $colPre+2, $rowPre3 + $charRow, $color2);
			imagesetpixel($pic, $colPre+3, $rowPre3 + $charRow, $color2);
			imagesetpixel($pic, $colPre+4, $rowPre3 + $charRow, $color3);
			imagesetpixel($pic, $colPre+5, $rowPre3 + $charRow, $color3);
			imagesetpixel($pic, $colPre+6, $rowPre3 + $charRow, $color4);
			imagesetpixel($pic, $colPre+7, $rowPre3 + $charRow, $color4);
		}
	}
}

// Output Image
imagepng($pic);

// Free Memory
imagedestroy($pic);
?>
